#!/usr/bin/env perl

#
#  Copyright (C) 2007 CeBiTec, Bielefeld University
#  Written by Wolfgang Gerlach.
#  The original version (CARMA 1.2) was developed by Lutz Krause.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  version 2 as published by the Free Software Foundation.
#
#  This file is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this file; see the file LICENSE.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.
#

use strict;
use DB;

use warnings;
no warnings 'recursion';



use FindBin;
use File::Basename;


use lib "$FindBin::Bin/../share/perl";


#use diagnostics;


use Getopt::Std;
use File::Temp qw(tempfile tempdir);

use TCommon;
use TCarma_phylo;
use TCarma_egt;

use Bio::SearchIO;
use IO::Handle;

use DNAUtils;
use TConfigs;


#######################################################################

my $OUTFILE = '';

$SIG{'INT'}  = 'handler';
$SIG{'TERM'} = 'handler';


my $INSTALLDIR = dirname($FindBin::Bin);
die "ERROR: INSTALLDIR variable not set in environment\n" unless $INSTALLDIR;

STDOUT->autoflush(1);









sub usage{
	print STDERR <<USAGE;
	
Copyright (C) 2007 CeBiTec, Bielefeld University.
Written by Lutz Krause.
This software comes with ABSOLUTELY NO WARRANTY; This is free
software, and you are welcome to redistribute it under
certain conditions.
Read the COPYRIGHT for details.


Predict taxonomic origins of translated reads (EGTs):

Usage: carma_classify.pl -o <c> <options> <fasta_file/directory>	
	where: fasta file is a fasta file containing all sequences
		with pfam hit OR a directory containing fasta files
		(with suffix ".out" !)
		with sequences that have a pfam hit (the directory that
		was given with the -o option for the search of conserved
		Pfam protein domains and families)
		Alternatively STDIN can be used.

	Options
	-o <c>	: output directory 
	-c <n>	: submit <n> jobs
	-i	: submit on idle queue
			(should ALWAYS be used on CeBiTec compute cluster)
	-f <n>	: only run for single family (e.g. PF05341)
	-n <c>	: run for all families with name matching
			regular expression (PERL) <c>
	-g <n>	: run for all families annotated with certain GO-term
			(e.g. 'photosynthesis')
	-h	: show this help message
	-t	: don't delete temporary files
	-v	: verbose
	(-j	: reserved for internal usage)

USAGE
}

#########################
# ERROR handler, catches INT and TERM signal
#########################
sub handler {
	my $signal = shift;	# signal-nummer besorgen
	$SIG{'INT'}  = 'handler';   # reinstall sig-handler
	$SIG{'TERM'} = 'handler';   # reinstall sig-handler

	print STDERR "Signal: SIG$signal caught!\n";
	if ($signal eq "INT" or $signal eq "TERM"){
		print STDERR "Bailing out ...\n";
		die;
	}
	else{
		print STDERR "Caught signal $signal, don\'t know what to do !!!\n";
	}
	return 1;
}



########################
# Fetch hmm-model of single
# Pfam family
########################
sub get_hmmfile{
	my ($acc) = @_;

	my (undef, $hmmfile) = tempfile (SUFFIX=> '.hmm_fs', OPEN => 0);
	
	#my $hmmfile = TMP_DIR . "/$acc.hmm_fs";

	#return $hmmfile if -e $hmmfile;

	# create file
	#open(T,">$hmmfile");
	#close(T);

	my $cmd = HMMFETCH . ' ' . PFAM_FS . " $acc > $hmmfile";
	
	unless(system($cmd) == 0){
		unlink($hmmfile) unless ($main::opt_t);
		print STDERR "ERROR: Could not execute $cmd\n$!\n";
		return;
	}
	return $hmmfile;
}



##################
# Get options
##################
our($opt_v,$opt_h,$opt_o,$opt_c,$opt_m,$opt_j,
	$opt_i,$opt_f,$opt_g,$opt_n, $opt_t);

getopts('vho:c:m:jif:g:n:t');


# print usage message
if($opt_h){
	usage;
	exit;
}


unless (CONSIDERED_RANKS_MODUS) {
		print STDERR "ERROR: Please set variable CONSIDERED_RANKS_MODUS in configuration file.\n";
		exit(1);
}

my $fasta_file;
my $fh;

if (@ARGV) {
	$fasta_file = shift(@ARGV);
} else {

	# check for data from STDIN

	
	if (-t STDIN) {
		main::usage();
		exit(1);
	}

		
	# move content of STDIN into temporary file
	
	($fh, $fasta_file) = tempfile (UNLINK => $main::opt_t?0:1,  DIR =>  TMP_DIR, SUFFIX=> '.fas');
		
	unless(open($fh,">$fasta_file")){
		print STDERR "ERROR: Could not open file $fasta_file:\n$!\n";
		die;
	}

	while (<STDIN>) {
		print $fh $_;
	}

	close($fh);	
}


my $single_seqs_with_pfam_hit = {};

  
TCarma_phylo_fasta::carma_phylo_fasta($fasta_file, $single_seqs_with_pfam_hit);



if ( $main::opt_j ) {
	########### JOBS
	TCarma_phylo_job::carma_phylo_job($single_seqs_with_pfam_hit);
} else {
	########### JOB SUBMISSION
	TCarma_phylo::carma_phylo($INSTALLDIR, $fasta_file, $single_seqs_with_pfam_hit);
}

exit(0);



