#!/usr/bin/env perl

#
#  Copyright (C) 2007 CeBiTec, Bielefeld University
#  Written by Wolfgang Gerlach.
#  The original version (CARMA 1.2) was developed by Lutz Krause.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  version 2 as published by the Free Software Foundation.
#
#  This file is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this file; see the file LICENSE.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.
#

use strict;
use DB;

use warnings;
no warnings 'recursion';

use FindBin;
use lib "$FindBin::Bin/../share/perl";
use File::Basename;

#use diagnostics;


use Getopt::Std;
use File::Temp qw(tempfile tempdir);

use TCommon;
#use TCarma_phylo;
use TCarma_egt;

use Bio::SearchIO;
use IO::Handle;

use DNAUtils;
use TConfigs;



$SIG{'INT'}  = 'handler';
$SIG{'TERM'} = 'handler';


my $INSTALLDIR = dirname($FindBin::Bin);




die "ERROR: INSTALLDIR variable not set in environment\n" unless $INSTALLDIR;

STDOUT->autoflush(1);


sub usage{
	print STDERR <<USAGE;
	
Copyright (C) 2007 CeBiTec, Bielefeld University.
Written by Lutz Krause.
This software comes with ABSOLUTELY NO WARRANTY; This is free
software, and you are welcome to redistribute it under
certain conditions.
Read the COPYRIGHT for details.


carma.pl: taxonomic classification of short environmental DNA fragments
	
Search for conserved Pfam protein domains and families (EGT's):

Usage: carma_search.pl -o <output_directory> <options> <fasta_file>
  where: fasta_file contains the metagenomic reads

	Options
	-o <c>	: output directory
	-c <n>	: submit <n> jobs
	-D <c>	: safe pfam output in this directory
	-i	: submit on idle queue
			(should ALWAYS be used on CeBiTec compute cluster)
	-e <n>	: hmmpfam evalue cut off
	-b <n>	: blast evalue cut off
	-h	: show this help message
	-t	: don't delete temporary files			 
	-v	: verbose
	(-j	: reserved for internal usage)


USAGE
}

#########################
# ERROR handler, catches INT and TERM signal
#########################
sub handler {
	my $signal = shift;	# signal-nummer besorgen
	$SIG{'INT'}  = 'handler';   # reinstall sig-handler
	$SIG{'TERM'} = 'handler';   # reinstall sig-handler

	print STDERR "Signal: SIG$signal caught!\n";
	if ($signal eq "INT" or $signal eq "TERM"){
		print STDERR "Bailing out ...\n";
		die;
	}
	else{
		print STDERR "Caught signal $signal, don\'t know what to do !!!\n";
	}
	return 1;
}



########################
# Fetch hmm-model of single
# Pfam family
########################
sub get_hmmfile{
	my ($acc) = @_;

	my (undef, $hmmfile) = tempfile (SUFFIX=> '.hmm_fs', OPEN => 0);
	
	#my $hmmfile = TMP_DIR . "/$acc.hmm_fs";

	#return $hmmfile if -e $hmmfile;

	# create file
	#open(T,">$hmmfile");
	#close(T);

	my $cmd = HMMFETCH . ' ' . PFAM_FS . " $acc > $hmmfile";
	
	unless(system($cmd) == 0){
		unlink($hmmfile) unless ($main::opt_t);
		print STDERR "ERROR: Could not execute $cmd\n$!\n";
		return;
	}
	return $hmmfile;
}



##################
# Get options
##################
our($opt_v,$opt_e,$opt_h,$opt_b,$opt_o,$opt_c,$opt_m,$opt_j,
	$opt_D,$opt_i,$opt_t);

getopts('e:vhb:o:c:m:jD:it');




# print usage message
if($opt_h){
	usage;
	exit;
}


my $fasta_file;
my $fh;

if (@ARGV) {
	$fasta_file = shift(@ARGV);
	
	if (-d $fasta_file) {
		print STDERR "ERROR: Don't accept directories as input.\n";
	}
} else {

	# check for data from STDIN

	
	if (-t STDIN) {
		main::usage();
		exit(1);
	}

		
	# move content of STDIN into temporary file
	
	($fh, $fasta_file) = tempfile (UNLINK => $main::opt_t?0:1,  DIR =>  TMP_DIR, SUFFIX=> '.fas');
		
	unless(open($fh,">$fasta_file")){
		print STDERR "ERROR: Could not open file $fasta_file:\n$!\n";
		die;
	}

	while (<STDIN>) {
		print $fh $_;
	}

	close($fh);	
}

if ($main::opt_o) {
	if($main::opt_o =~ /\.\./){
		print STDERR "ERROR: use absolut path names!\n";
		print STDERR "(-o option: $main::opt_o)\n";
		exit 1;
	}
}	



########################################
# Search for conserved protein domains #
########################################
if ( $main::opt_j ) {
	# jobs:
	
	TCarma_egt_job::carma_egt_job($INSTALLDIR, $fasta_file);
} else {

	
	# job submitter:
	TCarma_egt::carma_egt($INSTALLDIR, $fasta_file);
}
	

