package DNAUtils::Codes;
#
#  This file is part of GenDB 2.2
#  Copyright (C) 2005 CeBiTec, Bielefeld University
#                     <gendb@CeBiTec.Uni-Bielefeld.DE>
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  version 2 as published by the Free Software Foundation.
#
#  This file is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this file; see the file LICENSE.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.
#
=head1 NAME

DNAUtils::Code

=head1 DESCRIPTION

contains tables with codon translations and start/stop codons for the various genetic codes

=cut

    use strict;
use warnings;
use base qw(Exporter);

use constant CODON_TABLES => { 
    1 => { AAA => 'K', AAT => 'N', AAG => 'K', AAC => 'N',
	   ATA => 'I', ATT => 'I', ATG => 'M', ATC => 'I',
	   AGA => 'R', AGT => 'S', AGG => 'R', AGC => 'S',
	   ACA => 'T', ACT => 'T', ACG => 'T', ACC => 'T',
	   TAA => '*', TAT => 'Y', TAG => '*', TAC => 'Y',
	   TTA => 'L', TTT => 'F', TTG => 'L', TTC => 'F',
	   TGA => '*', TGT => 'C', TGG => 'W', TGC => 'C',
	   TCA => 'S', TCT => 'S', TCG => 'S', TCC => 'S',
	   GAA => 'E', GAT => 'D', GAG => 'E', GAC => 'D',
	   GTA => 'V', GTT => 'V', GTG => 'V', GTC => 'V',
	   GGA => 'G', GGT => 'G', GGG => 'G', GGC => 'G',
	   GCA => 'A', GCT => 'A', GCG => 'A', GCC => 'A',
	   CAA => 'Q', CAT => 'H', CAG => 'Q', CAC => 'H',
	   CTA => 'L', CTT => 'L', CTG => 'L', CTC => 'L',
	   CGA => 'R', CGT => 'R', CGG => 'R', CGC => 'R',
	   CCA => 'P', CCT => 'P', CCG => 'P', CCC => 'P' },
    4 => { AAA => 'K', AAT => 'N', AAG => 'K', AAC => 'N',
	   ATA => 'I', ATT => 'I', ATG => 'M', ATC => 'I',
	   AGA => 'R', AGT => 'S', AGG => 'R', AGC => 'S',
	   ACA => 'T', ACT => 'T', ACG => 'T', ACC => 'T',
	   TAA => '*', TAT => 'Y', TAG => '*', TAC => 'Y',
	   TTA => 'L', TTT => 'F', TTG => 'L', TTC => 'F',
	   TGA => 'W', TGT => 'C', TGG => 'W', TGC => 'C',
	   TCA => 'S', TCT => 'S', TCG => 'S', TCC => 'S',
	   GAA => 'E', GAT => 'D', GAG => 'E', GAC => 'D',
	   GTA => 'V', GTT => 'V', GTG => 'V', GTC => 'V',
	   GGA => 'G', GGT => 'G', GGG => 'G', GGC => 'G',
	   GCA => 'A', GCT => 'A', GCG => 'A', GCC => 'A',
	   CAA => 'Q', CAT => 'H', CAG => 'Q', CAC => 'H',
	   CTA => 'L', CTT => 'L', CTG => 'L', CTC => 'L',
	   CGA => 'R', CGT => 'R', CGG => 'R', CGC => 'R',
	   CCA => 'P', CCT => 'P', CCG => 'P', CCC => 'P' },
    11 =>{ AAA => 'K', AAT => 'N', AAG => 'K', AAC => 'N',
	   ATA => 'I', ATT => 'I', ATG => 'M', ATC => 'I',
	   AGA => 'R', AGT => 'S', AGG => 'R', AGC => 'S',
	   ACA => 'T', ACT => 'T', ACG => 'T', ACC => 'T',
	   TAA => '*', TAT => 'Y', TAG => '*', TAC => 'Y',
	   TTA => 'L', TTT => 'F', TTG => 'L', TTC => 'F',
	   TGA => '*', TGT => 'C', TGG => 'W', TGC => 'C',
	   TCA => 'S', TCT => 'S', TCG => 'S', TCC => 'S',
	   GAA => 'E', GAT => 'D', GAG => 'E', GAC => 'D',
	   GTA => 'V', GTT => 'V', GTG => 'V', GTC => 'V',
	   GGA => 'G', GGT => 'G', GGG => 'G', GGC => 'G',
	   GCA => 'A', GCT => 'A', GCG => 'A', GCC => 'A',
	   CAA => 'Q', CAT => 'H', CAG => 'Q', CAC => 'H',
	   CTA => 'L', CTT => 'L', CTG => 'L', CTC => 'L',
	   CGA => 'R', CGT => 'R', CGG => 'R', CGC => 'R',
	   CCA => 'P', CCT => 'P', CCG => 'P', CCC => 'P'}};

use constant START_CODONS => { 1 => [qw(ATG CTG TTG)],
			       4 => [qw(TTA TTG CTG ATG ATT ATC ATA GTG)],			       
			       # @@@ removed CTG !!!
			       11 => [qw(ATG GTG TTG)]};
#			       11 => [qw(ATG CTG GTG TTG)]};

use constant STOP_CODONS => { 1 => [qw(TAA TAG TGA)],
			      4 => [qw(TAA TGA)],
			      11 => [qw(TAA TAG TGA)]};

our @EXPORT_OK = qw(CODON_TABLES START_CODONS STOP_CODONS);
1;
