package TCarma_MatePairFunctions;


#
#  Copyright (C) 2007 CeBiTec, Bielefeld University
#  Written by Wolfgang Gerlach.
#  The original version (CARMA 1.2) was developed by Lutz Krause.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  version 2 as published by the Free Software Foundation.
#
#  This file is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this file; see the file LICENSE.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.
#



use strict;
use warnings;


use TCommon;

1;



#########################
# compute complementary mate name
#########################
sub getMateName{
	my ($query_name) = @_;
	# illumina format, example: "HWI-EAS217_1_2013P:1:95:916:131/2_1_2"
	my ($matePrefix, $mateNum);
	unless (($matePrefix, $mateNum) = $query_name =~ /^(\S+)(\d)_-?\d_-?\d$/ ) {
		die "ERROR: wrong name format \$query_name: ($query_name)\n";
	}
	if ($mateNum == 1) {
		  $mateNum = 2;
	} else {
		  $mateNum = 1;
	}
	# return, example: ( HWI-EAS217_1_2013P:1:95:916:131/1  , 1 )
	return \($matePrefix . $mateNum, $mateNum);
}


sub mergeMatePairs { #TODO generalize?
	my ($sequences) = @_;
	
	for(my $j = 0; $j < @$sequences; $j++){
		my $acc = @{$sequences}[$j]->[0];
	
		# check if sequence is a known member of pfam family
		next if TCommon::_is_pfam($acc); # TODO remove?

		my ($name) = $acc =~ /^\d+c_(.*)$/;
				
		my ($mateName) = getMateName($name);
		$mateName = $$mateName;

		#search for such a mate:
		for(my $i = $j+1; $i < @$sequences; $i++){
			my $accMate = @{$sequences}[$i]->[0];
			next if TCommon::_is_pfam($accMate); # TODO remove?
		
			my ($nameCheck) = $accMate =~ /^\d+c_(\S*\d)_-?\d_-?\d$/;
			unless ($nameCheck) {
				#next;
				print STDERR "ERROR (mergeMatePairs): not vaild read description \"$acc\". Should have been found by _is_pfam.";
				die;
			}

			

			if ($nameCheck eq $mateName) {
				#found mate !!!
				#now merge mates:
				#check for overlap (overlap would not be good)
				my $isMatePair=1;
				#my $newSequence="";
				my $maxLength = (length(@{$sequences}[$i]->[1]) > length(@{$sequences}[$j]->[1])) ? length(@{$sequences}[$i]->[1]) :length(@{$sequences}[$j]->[1]) ;
				my $newSequence= ("?" x  $maxLength);

				for (my $k=0; $k < length(@{$sequences}[$j]->[1]); $k++ ) {
				# merges rule: (?,?)->(?) ; (X,?)->(X) ; (X,-)->X  ;  (?,-)->(-) ; (-,-)->(-) ; (X,X)->(X)
				# problems: (X,!X) -> overlap!? (better do not allow this)
				
					my $a = substr(@{$sequences}[$i]->[1], $k, 1);
					my $b = substr(@{$sequences}[$j]->[1], $k, 1);
					
					if ( $a eq "?") { # (?,?)->(?) ; (X,?)->(X) ; (?,-)->(-)
						substr($newSequence,$k,1,$b);
					} 
					elsif ( $b eq "?") { 
						substr($newSequence,$k,1,$a);
					} 
					elsif ( $a eq $b) { # (-,-)->(-) ; (X,X)->(X), unlikely
						substr($newSequence,$k,1,$a);
						# TODO "substr outside of string" why ???
					
					}
					elsif ($a eq "-") {
						substr($newSequence,$k,1,$b);
					}
					elsif ($b eq "-") {
						substr($newSequence,$k,1,$a);
					}
					else { #overlap! (X,Y), where X!=Y
						$isMatePair=0;
					}
				}
				
				#print STDERR "\n$newSequence\n";
				if ($isMatePair == 1) {

					#check distance, should be around 13 AA, min=0, max=24 AA
					# or length: 13AA+24AA+13AA=50AA is maximum
					my ($newSequenceLength) = $newSequence =~ /\?*([A-Z]|\-]+)\?*/;
					$newSequenceLength =~ s/\-//;
					if (defined $main::opt_m) {
						
						if ( (length($newSequenceLength) >= $main::opt_m) && ($main::opt_m != -1) ) { # don't treat it as a mate-pair. TODO make a variable out of the 50!
							$isMatePair=0;
						}
					}
				}
				
				if ($isMatePair == 1) {
					my $newAcc = @{$sequences}[$j]->[0];
					# substitute "1" or "2" by "3", which indicates a mate-pair.
					$newAcc =~ s/^(\S*)\/(\d)_-?\d_-?\d$/$1\/3/; # geht das mit der 3 eleganter?
		
					# replace mate1 with newAcc/newSequence:
					@{$sequences}[$j]->[0] = $newAcc;
					@{$sequences}[$j]->[1] = $newSequence;
					# delete mate2 at $i:
					splice(@{$sequences}, $i, 1);
				} #end of if
				last; # do not need to continue search as we have already found the mate
			}#if
		}#for search mate in @sequences
		
	}# for @sequences	

}



