package TCarma_egt;
#
#  Copyright (C) 2007 CeBiTec, Bielefeld University
#  Written by Wolfgang Gerlach.
#  The original version (CARMA 1.2) was developed by Lutz Krause.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  version 2 as published by the Free Software Foundation.
#
#  This file is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this file; see the file LICENSE.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.
#

use strict;
use warnings;



#use Getopt::Std;
use File::Basename;
#use POSIX qw(tmpnam);
use File::Temp qw(tempfile tempdir);
#use File::Temp qw/ tempfile tempdir /;
use POSIX qw(ceil floor);

use FileUtils;

use DBI;
use TCommon;
use TCarma_egt_job;


use Tree;
use Bio::SearchIO;
use Cwd;
use IO::Handle;

use DNAUtils;

use TConfigs;

my ($INSTALLDIR, $CODINE_ERROR_LOG, $BLAST_ERROR_LOG);

1;









##################################
# Submit single job to run hmmpfam and BLAST
##################################
sub _submit_search{
	my ($submit,$scheduler_ref,$outfile, $outfile_pfam, $job_id) = @_;

	
	# use codine to submit job
	if(USE_CODINE){

		my $tmp_fasta_sequences="";
	
		foreach my $name_seq (@$submit){
			my ($name,$seq) = @$name_seq;
	
			#print $fh ">$name\n$seq\n";
			$tmp_fasta_sequences .= ">$name\n$seq\n";
		}

		require
			 Scheduler::DRMAA;	
		
		$scheduler_ref->add_input_data($job_id, $tmp_fasta_sequences);

	}
	# use user defined scheduler
	# to submit job
	else{
		my ($fh, $tmp_fasta) = tempfile (UNLINK => $main::opt_t?0:1,  DIR =>  TMP_DIR, SUFFIX=> '.fas');
	
		unless(open($fh,">$tmp_fasta")){
			print STDERR "ERROR: Could not open file $tmp_fasta:\n$!\n";
			die;
		}

		foreach my $name_seq (@$submit){
			my ($name,$seq) = @$name_seq;
	
			print $fh ">$name\n$seq\n";
		}


		close($fh);
		
		


		my $optD = '';
		if($outfile_pfam){
			$optD = "-D $outfile_pfam ";
		}
		
		my $optt = ($main::opt_t?"-t":"");
		my $optv = ($main::opt_t?"-v":"");

		my($filename, $directories, $suffix) = fileparse($outfile, qr/\.[^.]*/);
		my $error_file=$directories."/".$filename.".log";
		
		my $cmd = $INSTALLDIR."/bin/carma_search.pl -j $optt $optv $optD -o $outfile $tmp_fasta 2>> $error_file";

	

		$cmd = SCHEDULER_SUBMIT . " $cmd";
	
		unless(system($cmd) == 0){
			print STDERR "ERROR while submitting job\n($cmd)\n";
			exit 1;
		}
	}
	
	return 1;
}


##################################
# Submit jobs to run hmmpfam and BLAST in parallel
##################################
sub _submit_search_jobs{
	my ($name,$seq,$submit,$job_id,$submitted,$seq_per_job,$prefix,$scheduler_ref,$opto,$dir,$limit) = @_;
	
	push @$submit,[$name,$seq];

	return 1 if $limit and $$job_id >= $limit;
	
	#return 1 if $$submitted > 10;

	# submit job
	if(@$submit >= $seq_per_job){

		
#		if (defined $main::opt_l) {
#			$submit_job=0;
#			my @jobs=split(/,/, $main::opt_l);
#			foreach my $job (@jobs) {
#				if ($job == $$job_id) {
#					$submit_job=1;
#					last;	
#				}
#			}
#		}

		#$$counter++;
		#my $outfile = "$opto/${prefix}_$$counter.out";
		my $outfile = "$opto/${prefix}_$$job_id.out";
#print STDERR "hallo";
		# submit job if outfile does not exist yet.
		unless (-e $outfile) {
		
			
			my $outfile_pfam = "$dir/${prefix}_$$job_id.pfam" if $dir;

			_submit_search($submit,$scheduler_ref,$outfile,$outfile_pfam,$$job_id);
			$$submitted++;
			print STDERR "\rSubmitting job ... $$job_id  ";
			
		}
		else{
			print STDERR "Skipping job, file $outfile exist!\n";
		}
		$$job_id++;
		@$submit = ();
	}
	
	return 1;
}








########################################
# Search for conserved protein domains #
########################################
sub carma_egt{
	my $fasta_file;
	
	($INSTALLDIR, $fasta_file) = @_;
	
	#print "carma_egt\n";		
	

	
#	unless($main::opt_o){
#		main::usage();
#		print STDERR "ERROR: -o option missing!\n";
#		exit 1;
#	}

	
	#my ($fasta_file) = @ARGV;
	 
	my $seq_per_job;
	
	
	
	
		###############################################
		# Submit search jobs 
		###############################################

		unless($main::opt_o){
			main::usage();
			print STDERR "ERROR: -o option missing!\n";
			exit 1;
		}

		unless(-d $main::opt_o){
			main::usage();
			print STDERR "ERROR: $main::opt_o is not a directory!\n";
			exit 1;
		}

		my $outdir = "$main::opt_o/". basename($fasta_file);
	
		unless($outdir =~ /^\//){
			print STDERR "ERROR: relative pathes not permitted (-o options)!\n";
			exit 1;
		}

		if($outdir =~ /\.\./){
			print STDERR "ERROR: relative pathes not permitted (-o options)!\n";
			exit 1;
		}

		# check of $outdir is a directory and exists, if not, create it
		unless(-d $outdir){
			unless(mkdir($outdir)){
				print STDERR "ERROR: Could not create directory $outdir!\n($main::opt_o)\n";
				exit 1;
			}
		}

		# check if $main::opt_D is a directory
		if($main::opt_D){
			die "ERROR: $main::opt_D not a directory!\n" unless -d $main::opt_D;
			
			
		}
	
	
	
	
	
	
		
		my $seq_number = 0;

		# Count sequence number in fasta file
		die "ERROR while parsing fasta file $fasta_file!!!\n" unless 
			&TCommon::parse_fasta($fasta_file,
						 sub {
							 $seq_number++;
							 return 1;
						 });

		# number of sequences per job
		print STDERR "Total number of sequences: ".$seq_number."\n";
		
		if ($seq_number == 0) {
			print STDERR "ERROR: No FASTA-Sequences found!\n";
			exit(1);
		}
		
		unless (defined $main::opt_c) {
			$main::opt_c=100;
		}
		
		if ($main::opt_c < 1) {
			die "Option -c has to be >= 1\n";
		}

		my $prefix = basename($fasta_file) . "_$main::opt_c";
		
				
		$seq_per_job = int($seq_number / $main::opt_c);

		
		print STDERR "Submitting (around) $main::opt_c jobs, $seq_per_job sequences per job ... \n";	
		

		


		my $scheduler;
		my $tmp_dir;
		if(USE_CODINE){
			
			require
				 Scheduler::DRMAA;		
		
			
			
				
			my @job_cmd_array;
			#my $perl_binary = `which perl`;
			#chomp($perl_binary);
			#unless ($perl_binary) {
			#	print STDERR "\"which perl\" did give no result !?\n";
			#	die;
			#}
			
			#push(@job_cmd_array, $perl_binary );
			push(@job_cmd_array, $INSTALLDIR."/bin/carma_search.sh" );
		 	push(@job_cmd_array, "-j" );
		 	if (defined $main::opt_t) {
		 		push(@job_cmd_array, "-t" );
		 	}
			if(defined $main::opt_b){
				push(@job_cmd_array, "-b $main::opt_b");
			}
			if(defined $main::opt_e){
				push(@job_cmd_array, "-e $main::opt_e");
			}

			
			$tmp_dir = FileUtils::get_tmp_dir_object(TMP_DIR);

			my $errorfile="$outdir/${prefix}_".$Scheduler::DRMAA::DRMAA_PLACEHOLDER_INCR.".log";
			
			
			$scheduler = Scheduler::DRMAA->new(temp_dir => $tmp_dir->dirname, commandline => [@job_cmd_array], error_file => $errorfile );
			
			
			
			if ($main::opt_i) {
		 			$scheduler->set_native_option('-l idle=1 ' . CODINE_OPTIONS);
			} else {
				$scheduler->set_native_option(CODINE_OPTIONS);
			}
			
			
		}
	
	
		my $job_id = 1;
		my $submitted=0;
		my @submit = ();
		
		# parse fasta file and submit jobs
		die "ERROR while parsing fasta file $fasta_file!\n" unless 
			&TCommon::parse_fasta($fasta_file, \&_submit_search_jobs,
						 \@submit,
						 \$job_id,
						 \$submitted,
						 $seq_per_job,
						 $prefix,
						 $scheduler,
						 $outdir,
						 $main::opt_D);
	
		# the rest that has not been submitted before
		if(@submit){
			my $outfile = "$outdir/${prefix}_$job_id.out";

			unless (-e $outfile) {
				my $outfile_pfam = "$main::opt_D/${prefix}_$job_id.pfam" if $main::opt_D;
				&_submit_search(\@submit,$scheduler,$outfile,$outfile_pfam, $job_id);
				
				$submitted++;
			} else {
				print STDERR "Skipping job, file $outfile exist!\n";
			}
			$job_id++;
			@submit = ();
		}

		if(USE_CODINE && $submitted > 0){
			$scheduler->submit();


			print STDERR "Done.\nSubmitted $submitted jobs!\n";
				
			my @error_jobs;
				
			my $success_jobs = 0;
			print STDERR "waiting and parsing results...\n";
			$scheduler->iterate(
				sub {
					my ($name, $fh) = @_;
					#print "Name: $name\n";
					#return unless ($all_jobs_ok);
				
					my $outfile = "$outdir/${prefix}_$name.out";
					my $out_fh;
					unless(open($out_fh,">$outfile")){
						print STDERR "ERROR: Could not open file $outfile:\n$!\n";
						die;
					}
				
					while (<$fh>) {
						print $out_fh $_;
					}
					close ($out_fh);
					
					$success_jobs++;
					print STDERR "Job \"" . $name . "\" finished successfully.\n";
					if ($submitted != (@error_jobs+$success_jobs)) {
						print STDERR ($submitted-(@error_jobs+$success_jobs)) . " are left.\n";
					}
				
				},
				sub {
					printf STDERR "Job %s failed: %s\n",$_[0],$_[1];

					push (@error_jobs, $_[0]);
					
				});
				
			if (@error_jobs > 0) {
				print STDERR "Sorry, some jobs did fail:\n";
				print STDERR "Jobs that failed: " . join(',', @error_jobs ) . "\n";
				print STDERR "Simply rerun CARMA with same parameters, only jobs that failed will be resubmitted!\n";
				print STDERR "(Will not work if called via carma.pl using option \"-o\"!)\n";
				exit(1);
			} else {
				print STDERR "All submitted jobs finished successfully!\n";
			}
			 # print STDERR "Jobid: $codine_job_id\n";
			exit(0);
			
		}
		exit 0;
		#} # if opt_c
	
	
}
