package TCarma_egt_job;
#
#  Copyright (C) 2007 CeBiTec, Bielefeld University
#  Written by Wolfgang Gerlach.
#  The original version (CARMA 1.2) was developed by Lutz Krause.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  version 2 as published by the Free Software Foundation.
#
#  This file is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this file; see the file LICENSE.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.
#

use strict;
use warnings;

#use lib '/homes/wgerlach/carma/share/perl';

#use Getopt::Std;
use File::Basename;
#use POSIX qw(tmpnam);
use File::Temp qw(tempfile tempdir);
#use File::Temp qw/ tempfile tempdir /;

use DBI;
use TCommon;



use Tree;
use Bio::SearchIO;
use Cwd;
use IO::Handle;

use DNAUtils;

use TConfigs;

my ($INSTALLDIR, $CODINE_ERROR_LOG, $BLAST_ERROR_LOG);

1;


#####################################
# Add environmental DNA fragment with
# BLAST hit to hash of BLAST supported
# fragments
#####################################
sub pfam_hit{
	my ($unit,$processed,$hmmacc,$seqs,$hitting_seqs) = @_;

	my $seqname = $unit ->{seqname};

	return 1 if exists $processed->{$seqname};
	
	$processed->{$seqname} = 1;

	my $query_seq = $seqs->{$seqname};

	die unless $query_seq;

	my $hmmname = $unit->{hmmname};
	my $query_from = $unit ->{start_seq};
	my $query_end = $unit ->{end_seq};
	my $evalue = $unit ->{evalue};
	

	
	my $family_info = $unit->{family};
	
	my $subject_from = $unit ->{start_hmm};
	
	my $hitting_seq = substr($query_seq ,$query_from - 1,$query_end - $query_from + 1);

   
	return 1 unless length($hitting_seq) >= MINIMUM_PFAM_HIT_LENGTH;
	
	push @{ $hitting_seqs->{$hmmacc}},[$seqname,$hitting_seq,$subject_from-$query_from,$evalue];

	return 1;
}

##############################
# Resolve frame shifts predicted
# by blastx
##############################
sub get_seq_with_frame_shift{
	my ($qseq,$seqs,$query,$strand,$frame,$start,$end) = @_;

	die "ERROR: query $query does not exist in fasta file!\n".(keys %$seqs)."\n" unless exists $seqs->{$query};

	my $insertions = $qseq =~ tr/\\/\\/;
	my $deletions = $qseq =~ tr/\//\//;

	my $in_dels = $insertions + $deletions;
	
	$frame = -$frame if $frame < 0;	#1,2,3
	
	my $qseq_full = $seqs->{$query};
	
	my $frame_offset = $frame - 1;

	my $full_seq = '';
	
	if($strand > 0){	
		# get leading sequence
		my $leading = substr($qseq_full,$frame_offset,$start + 9 - $frame_offset - 1);
	
		die "ERROR: wrong length $query\n" if length($leading) % 3;

		# translate leading sequence
		$leading = DNAUtils::translate($leading);
	
		$leading =~ s/([\w\*]{3})$// or die "ERROR: no leading $query\n";

		my $leading_overlap = $1;
	
		$leading_overlap =~ s/\*/\\\*/g;
		
		# TODO this check is a problem when containing an X
		#unless($qseq =~ /^$leading_overlap/){
		#	my $tmp = substr($qseq,0,3);
		#
		#	die "ERROR: $query leading: ($leading) start: $start frame: $frame $query overlap: $leading_overlap\n($qseq)\n" unless ($tmp =~ /\\/ or $tmp =~ /\//);
		#}

	
		# get suffix
		my $suffix = substr($qseq_full,$end - 9);

		$suffix = DNAUtils::translate($suffix);

		unless($suffix =~ s/^([\w\*]{3})//){
			my $tmp = substr($qseq,-3);

			die "ERROR: no suffix: $query\n($suffix , $tmp)\n";
		}
		my $suffix_overlap = $1;

		$suffix_overlap =~ s/\*/\\\*/g; # for the regexp-matching

		# TODO this check is a problem when containing an X
		#unless ($qseq =~ /$suffix_overlap$/){
		#	my $tmp = substr($qseq,-3);
		#	die "ERROR: query: $query\nqseq: $qseq\nsuffix: $suffix\nsuffix_overlap: $suffix_overlap\n" unless ($tmp =~ /\\/ or 
		#								$tmp =~ /\// or 
		#								$tmp =~ /\*/);
		#}
	
		$full_seq = "${leading}${qseq}${suffix}";
	} else{
		my $qseq_full_rc = DNAUtils::reverse_complement($qseq_full);
	
		my $length = length($qseq_full_rc);
		
		$start = $end - (( (length($qseq) - ($in_dels)) * 3) - $insertions + $deletions) + 1;

		my $tmp = $start;
		$start = $length - $end + 1;
	

		$end = $length - $tmp + 1;
	
		my $prefix_frame_offset = $frame - 1;

		my $prefix = substr($qseq_full_rc,$prefix_frame_offset,$start + 9 - $prefix_frame_offset - 1);
	
		die "ERROR: wrong prefix length!!! $query\n" if length($prefix) % 3;

	

		die "ERROR: $query $frame length ".length($prefix)." start: $start offset: $prefix_frame_offset\n" if length($prefix) % 3;

		$prefix = DNAUtils::translate($prefix);


		$prefix =~ s/([\w\*]{3})$// or die "ERROR: no prefix $query ($prefix)\n";
	
		#my $prefix_overlap = $1;
	
		#$prefix_overlap =~ s/\*/\\\*/g;
		# TODO this check is a problem when containing an X
		#unless ($qseq =~ /^$prefix_overlap/){
		#	my $tmp = substr($qseq,0,3);
		#	die "ERROR: prefix $query $prefix $prefix_overlap\n($qseq)\n" unless ($tmp =~ /\\/ or $tmp =~ /\// or $tmp =~ /\*/ or $prefix_overlap =~ /X/);
		#}
		my $suffix_frame = $end % 3;
		$suffix_frame++;
	

		my $suffix = substr($qseq_full_rc,$end - 9);

		$suffix = DNAUtils::translate($suffix);

		$suffix =~ s/^([\w\*]{3})// or die "ERROR: $query suffix must at least have 3 AAs! ($suffix)\n";

		#my $suffix_overlap = $1;

		#$suffix_overlap =~ s/\*/\\\*/g; # for the regexp-matching


		# TODO this check is a problem when containing an X	
		#unless ($qseq =~ /$suffix_overlap$/){
		#	my $tmp = substr($qseq,-3);
		#	unless ($tmp =~ /\\/ or $tmp =~ /\// or $tmp =~ /\*/){
		#
		#		print translate($qseq_full_rc),"\n\n";
		#
		#		foreach (1,2){
		#			print STDERR DNAUtils::translate(substr($qseq_full_rc,$_)),"\n\n";
		#		}
		#
		#		die "ERROR: $query suffix no overlap $strand $frame $suffix_overlap\n($qseq)\n"; 
		#	}
		#}

		$full_seq = "${prefix}${qseq}${suffix}";
	}
	die "ERROR: full seq contains - $query\n($full_seq)\n" if $full_seq =~ /-/;
	
	$full_seq =~ tr/\*/Z/;
	$full_seq =~ s/\\//g;
	$full_seq =~ s/\///g;

	return $full_seq;
}



##########################
# Get sequence for a certain reading frame
###########################
sub get_seq{
	my ($qseq,$seqs,$query,$strand,$frame) = @_;
	
	die "ERROR: query $query does not exist in fasta file!\n".(keys %$seqs)."\n" unless exists $seqs->{$query};
	
	my $qseq_full = $seqs->{$query};

	$frame = -$frame if $frame < 0;

	if($strand < 0){
		$qseq_full = reverse_complement($qseq_full);
	}
	if($frame > 1){
		$qseq_full = substr($qseq_full,$frame - 1);
	}
	$qseq_full = translate($qseq_full);
	$qseq_full =~ tr/\*/Z/;
	
	#my $tmp_qseq = $qseq;

	#$tmp_qseq =~ s/\*/\\\*/g;

	# TODO gives warnings if qseq_full contains a X
	#print STDERR "WARNING: $query strand: $strand get_seq: $qseq_full ($qseq)\n" unless $qseq_full =~ /$tmp_qseq/;

	return $qseq_full;
}



################################
# Run blast search of query multiple 
# fasta file against Pfam's
# underlying sequence database
# Candidate members of pfam families
# and frame-shifts are identified
################################
sub run_blast{

	my ($fasta_file,$evalue_blast,$seqs) = @_;


	#my $cmd = BLASTALL . " -p blastx -w 15 -i $fasta_file -d " . PFAM_A_FASTA . " -e $evalue_blast -F F -b 20 -v 20 2>> ".$BLAST_ERROR_LOG;
	my $cmd = BLASTALL . " -p blastx -w 15 -i $fasta_file -d " . PFAM_A_FASTA . " -e $evalue_blast -F F -b 20 -v 20";
	unless(open BLAST, "$cmd |"){
		print STDERR "ERROR while running $cmd:\n$!\n";
		return;
	}
		
	#########################
	# Prarse blast output
	#########################
	my $searchin = Bio::SearchIO -> new( '-fh'   => \*BLAST,
					 '-format' => 'Blast' );

	my %result = ();

	my %query_by_pfam_family = ();
	while( my $result = $searchin->next_result() ) {

		my $query_name = $result->query_name();

		while(my $hit = $result -> next_hit()){

			my $descr = $hit->description;

			unless($descr){
				print STDERR "WARNING: no hit description!\n" if WARNINGS;
				next;
			}

			my( $acc, $id );
			unless( ( $acc, $id ) = $descr =~ /(PF\d+\.?\d*)\;(\S+)\;/){
				print STDERR "WARNING: wrong description format ($descr)\n" if WARNINGS;
				next;
			}

			#my( $acc, $id ) = ($1,$2);
		
			my $hsp = $hit->next_hsp();
		
			next unless $hsp;
		
			#my $evalue = $hsp->evalue();
		
			my $qseq = $hsp->query_string;
		
			if($qseq =~ /^\s*$/ or ! $qseq){
				print STDERR "WARNING: no query sequence!!!!!\n";
				next;
			}

			my $rank = $hsp->rank;
		
			my $strand = $hsp->strand();
			my $frame = $hsp->frame();
		
			$frame++;
			if($strand < 0){
				$frame = - $frame;
			}
		
			my $seq_id = "${query_name}_${strand}_${frame}" ;
		
			next if exists $query_by_pfam_family{$acc}->{$query_name};
			die if exists $result{$acc}->{q}->{$seq_id};
		
		
			my $start = $hsp->start('query');
			my $end = $hsp->end('query');
		
			my @frame_shifts = ();
		
			$qseq =~ s/\-//g;

			my $seq_with_hit = '';

			#######################################
			# Resolve predicted frame-shifts
			#######################################
			# BLAST found local alignments. We want the reads to be translated completely:
			if($qseq =~ /\// or $qseq =~ /\\/){			# "/" 1 nucleotide insertion - "frameshift +1"
				#print STDERR "frameshift=1\n";
				$seq_with_hit = get_seq_with_frame_shift($qseq,
									 $seqs,
									 $query_name,
									 $strand,
									 $frame,
									 $start,
									 $end);

				die "ERROR: $query_name\n" unless $seq_with_hit;
			}
			else{
				#print STDERR "frameshift=0\n";
				$seq_with_hit = get_seq($qseq,$seqs,$query_name,$strand,$frame);
				
				die "ERROR: $query_name\n" unless $seq_with_hit;
			}
		
			#$result{$acc}->{id} = $id;
		
			#$result{$acc}->{q}->{$seq_id} = $seq_with_hit;		
			$result{$acc}->{$seq_id} = $seq_with_hit;
			
			$query_by_pfam_family{$acc}->{$query_name} = 1;

		}	

		delete $seqs->{$query_name};
	}

	unless (close(BLAST)) {
		print STDERR "ERROR: could not close BLAST \"$!\"\n"; 
		print STDERR "ERROR: called: \"$cmd\"\n";		
		exit(1);
	}


		
	return \%result;
}


###################################
# Parse hmmpfam output file
##################################
sub parse_hmmpfam {
	my $file = shift;
	my $sub_ref = shift;

	
  QUERY: {
		my( %hash, $seqname );
		while(<$file>) {
			/^Scores for sequence/ && last;
			if( /^Query sequence:\s+(\S+)/ ) {
				$seqname = $1;
			}
		}
		  
		while(<$file>) {
			next if /^Model\s+Description\s+Score/;
			next if /\[no hits above thresholds\]/;
			next if /^\s*$/;
			next if /^--------/;

			/^Parsed for domains/ && last;
			if( my( $id, $descr, $score, $evalue ) = /^\s*(\S+)\s+(\S+.*?)\s+([-\d\.]+)\s+(\S+)\s+\d+$/ ) {

				  $hash{$id} = [$descr,$score, $evalue];
			} 
			else{
				  die "ERROR: 1 $seqname wrong line format ($_)\n";
			}
		}
		  
		while(<$file>) {
			  next if /\[no hits above thresholds\]/;
			  next if /Model\s+Domain\s+seq-f\s+seq-t/;
			  next if /^--------/;
			   /^\/\// && redo QUERY;
			  
			  if( my ( $id, $sqfrom, $sqto, $hmmf, $hmmt, $sc, $ev ) = 
				  /(\S+)\s+\S+\s+(\d+)\s+(\d+).+?(\d+)\s+(\d+)\s+\S+\s+(\S+)\s+(\S+)\s*$/ ) {
	
				  unless(exists($hash{$id}) ) {
					  # hmmpfam seems to return hits above the domain score that aren't
					  # above the sequence score.  We don't want these, so skip if we
					  # haven't seen a sequence score.
					  next;
			#		  warn("HMMResults parsing error in hmmpfam for $id - can't find sequence score");
				  }
				  
				  my $unit = {};
				  
				  
				  $unit ->{seqname} = $seqname;
				  $unit ->{hmmname} = $id;
				  $unit ->{start_seq} = $sqfrom;
				  $unit ->{end_seq} = $sqto;
				  $unit ->{start_hmm} = $hmmf;
				  $unit ->{end_hmm} = $hmmt;
				  $unit ->{bits} = $sc;
				  $unit ->{evalue} = $ev;
				  $unit->{family} = $hash{$id};

				  # this should find it's own sequence!
				  
				  &$sub_ref($unit,@_);

			  }
			  else{		  
				  die "ERROR: 2 $seqname wrong line format\n($_)\n";
			  }
		  }
	} # end QUERY
	return 1;
}



###################################
# Run hmmpfam for candidate DNA fragments 
###################################
sub run_hmmpfam{
	my ($result,$evalue_pfam,$outfile,$verbose, $all) = @_;

	my %hitting_seqs_pfam = ();

	my ($acc,$hit_info);
	my $hits;
	
	my $run_pfam_counter = 0;
	
	############################
	# Iterate over each hitting Pfam family
	# In each iteration,
	# conduct hmmpfam search on all DNA fragments
	# that have a BLAST hit to the current family
	############################
	while ( ($acc,$hits) = each %$result){

		#my $id = $hit_info->{id};
		#my $hits = $hit_info->{q};
		


		unless(keys %$hits){
			print STDERR "ERROR: (run_hmmpfam) no hits!!!!\n";
			next;
		}

		#############################
		# skip hits to families with to many members
		#############################
		if(MAX_FAMILY_SIZE){
			my $tacc = $acc;
			$tacc =~ s/\.\d+$//;
		
			#################################################
			# Count number of sequences in family alignment
			#################################################
			my $tfile = PFAM_FAMILY_ALIGNMENTS . "/$tacc.fas";

			unless(-e $tfile){
				print STDERR "WARNING: file $tfile does not exist!!!!\n" if WARNINGS;
				next;
			}
			unless (defined $all) {
				my $cmd = "wc $tfile";

				open(T,"$cmd |" ) or die "ERROR: could not run $cmd:\n$!\n";
		
				my $line_nr;
				while(<T>){
					die unless /(\d+)\s+\d+\s+\d+/;  #/
					$line_nr = $1;

				}
				close(T);
			
			

				# Skip hit if familiy has to many members
			
				if ($line_nr > MAX_FAMILY_SIZE){
					print STDERR "WARNING: skipping hit to $acc, too many members ($line_nr)\n";
					next;
				}
			}
		}


		my ($fh, $tmp_fasta_file) = tempfile (UNLINK => $main::opt_t?0:1);
		
		
		# old: my $tmp_fasta_file = TMP_DIR . basename(tmpnam());
	

		#####################
		# Run hmmpfam for
		# matching DNA fragments
		#####################
		#unless(open(TMP_FASTA,">$tmp_fasta_file")){
		#	print STDERR "ERROR: Could not open fasta file $tmp_fasta_file:\n$!\n";
		#	die;
		#}
			
		eval{		
			my ($name,$seq);
		
			while (($name,$seq) = each %$hits){
				die "ERRPOR: (run_hmmpfam) no sequence!!!!@\n" unless length($seq);
				print $fh ">$name\n$seq\n";
				#print TMP_FASTA ">$name\n$seq\n";
			}
			#close(TMP_FASTA);
			close($fh);
		
			my $options = " -E $evalue_pfam ";
		
			# get hmm-model for current Pfam family
			my $hmmfile = main::get_hmmfile($acc);
		
			die unless $hmmfile;

			####################
			# run hmmpfam
			####################
			my $cmd = HMMPFAM." -A 0 $options $hmmfile $tmp_fasta_file"; 
			
		
			#####################################
			# Run hmmpfam and save results
			#####################################
			if($outfile){
				$run_pfam_counter++;
				my $pfamout = "$outfile.$run_pfam_counter";

				$cmd .= " > $pfamout";
				#my $hmmpfam_time = time;
				
				unless(system($cmd) == 0){
					print STDERR "ERROR: Could not execute $cmd:\n$!\n";
					next;
				}
				
				unlink($hmmfile) unless ($main::opt_t);
				
				#$hmmpfam_time=time-$hmmpfam_time;
				#print STDERR "hmmpfam_time: $hmmpfam_time\n";
				
				my %processed = ();
		
				die unless $acc;

				###############################
				# Parse hmmpfam output file
				###############################
				unless(open PFAM_OUT, "<$pfamout"){
					print STDERR "ERROR: Could not open $pfamout\n$:\n";
					die;
				}
				
				die unless parse_hmmpfam(*PFAM_OUT,\&pfam_hit,\%processed,$acc,$hits,\%hitting_seqs_pfam);		
				
				close(PFAM_OUT);
			}
		
			#####################################
			# Run hmmpfam without saving results
			#####################################
			else{
			#my $hmmpfam_time=time();
			
			unless(open PFAM_OUT, "$cmd |"){
				print STDERR "ERROR: Could not execute $cmd\n$:\n";
				next;
			}
			
			#$hmmpfam_time=time()-$hmmpfam_time;
			#print STDERR "hmmpfam_time: $hmmpfam_time\n";
			
			my %processed = ();
		
			die unless $acc;

			die unless parse_hmmpfam(*PFAM_OUT,\&pfam_hit,\%processed,$acc,$hits,\%hitting_seqs_pfam);
			
			
			close(PFAM_OUT);
			}
		};
	
		# Check if an error occured
		if($@){
			unlink($tmp_fasta_file) unless ($main::opt_t);
			print STDERR "\n\n\nERROR running hmmfpam for $acc, $tmp_fasta_file:\n$@\n";
			die;
		}
		unlink($tmp_fasta_file) unless ($main::opt_t);
	}
	return \%hitting_seqs_pfam;
}



#######################################
# Print information on identified Pfam domains
#######################################
sub print_seqs_with_pfam_hit{
	my ($seqs_with_pfam_hit,$output_file,$verbose) = @_;
	my $outfile_workingname;

	if ($output_file) {
		print STDERR "Exporting sequences with pfam hit to file $output_file ... " if $verbose;
		$outfile_workingname = $output_file."part";
		unless(open(FA,">$outfile_workingname")){
			print STDERR "ERROR: Could not open file $output_file:\n$!\n";
			die;
		}
	}

	my ($pfam_hmm_acc,$hitting_seq_info);
	while ( ($pfam_hmm_acc,$hitting_seq_info) = each %$seqs_with_pfam_hit) {
		my ($pfam_acc_without_version, undef ) = split(/\./,$pfam_hmm_acc);

		my $go_ids_array =  TCommon::get_go_id_by_pfam_acc($pfam_acc_without_version);
		my $go_ids = "{".join(",",@$go_ids_array)."}";
	
		foreach my $hitting_seq (@$hitting_seq_info){		
			my ($query_name,$query_seq,undef,$evalue) = @$hitting_seq;
			
			
			my $outputlines = ">$pfam_hmm_acc=+=$query_name=+=$evalue=+=$go_ids\n$query_seq\n";
			
			if ($output_file) {			
				print FA $outputlines;
			} else {
				print STDOUT $outputlines;
			}
		}  
	}
	if ($output_file) {
		close(FA);
		#now rename the file 
		unless (rename($outfile_workingname, $output_file)) {
			print STDERR "ERROR: Could not rename $outfile_workingname to ". $output_file ." !?\n";
		}

	}
	
	print STDERR "Done.\n" if $verbose;
}





sub carma_egt_job{
	my $fasta_file;
	($INSTALLDIR, $fasta_file) = @_;
	



	####################################################
	# Search for conserved domains locally
	####################################################
	if($main::opt_D){
		die if -d $main::opt_D;
	}

	my $blast_evalue = defined $main::opt_b ? $main::opt_b : BLAST_DEFAULT_EVALUE;
	my $hmmpfam_evalue = defined $main::opt_e ? $main::opt_e : HMMPFAM_DEFAULT_EVALUE;

	my %all_sequences = ();

	# parse fasta file
	unless(TCommon::parse_fasta($fasta_file, sub { my ($name,$seq) = @_;  
						  $all_sequences{$name} = $seq; 
						  if (length($seq) == 0)  {
						  	die "ERROR: no sequence for \"$name\"!!!\n";
						  }
						  return 1;
						  }))
	{
		print STDERR "ERROR: while running for fasta file $fasta_file!\nBailing out ...\n";
		exit 1;
	}



	# run blast as a preprocessor in order to identify frame shifts in to find 
	# domain candidates
	print STDERR "Running blast ... " if $main::opt_v;
	
	#my $run_blast_time=time();
	my $blast_result = &run_blast($fasta_file,$blast_evalue,\%all_sequences);
	
	#$run_blast_time=time()-$run_blast_time;
	#print STDERR "run_blast_time: $run_blast_time\n";
	
	unless(ref $blast_result){
		print STDERR "\nERROR: while running blast on $fasta_file!\nBailing out ...\n";
		exit 1;
	}
	print STDERR "Done\n" if $main::opt_v;
	
	# remove fasta file
	unless ($main::opt_t) {unlink $fasta_file};
	
	
	# Run hmmpfam on all BLAST supported DNA fragments
	print STDERR "Running hmmpfam ... " if $main::opt_v;
	my $seqs_with_pfam_hit = &run_hmmpfam($blast_result,$hmmpfam_evalue,$main::opt_D,$main::opt_v);

	
	
	unless(ref $seqs_with_pfam_hit){
		print STDERR "\nERROR: while running hmmpfam!\nBailing out ...\n";
		exit 1;
	}
	print STDERR"Done\n" if $main::opt_v;
	
	# print results
	&print_seqs_with_pfam_hit($seqs_with_pfam_hit,$main::opt_o,$main::opt_v);
}

