package TCarma_phylo;
#
#  Copyright (C) 2007 CeBiTec, Bielefeld University
#  Written by Wolfgang Gerlach.
#  The original version (CARMA 1.2) was developed by Lutz Krause.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  version 2 as published by the Free Software Foundation.
#
#  This file is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this file; see the file LICENSE.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.
#

use strict;
use warnings;

#use lib '/homes/wgerlach/carma/share/perl';

#use Getopt::Std;
#use File::Basename;
#use POSIX qw(tmpnam);
use File::Temp qw(tempfile tempdir);
#use File::Temp qw/ tempfile tempdir /;

use DBI;
use TCommon;



use Tree;
use Bio::SearchIO;
use Cwd;
use IO::Handle;
use File::Basename;
    
use DNAUtils;

use TConfigs;
use TCarma_phylo_fasta;
use TCarma_phylo_job;

my ($INSTALLDIR, $CODINE_ERROR_LOG, $BLAST_ERROR_LOG);

1;






#########################
# add sequence with pfam hit to hash %seqs_with_pfam_hit
# for each pfam family this hash stores all DNA fragments matching the family
#########################
sub _add_seq_with_pfam_hit{
	my ($name,$seq,$seqs_with_pfam_hit,$family,$go_term,$id_regx,$verbose) = @_;

	return 1 if ($family and ! ($name =~ /$family/) );

	my $pfam_acc = "??";
	unless($name =~ /(PF\d+)/){
		print STDERR "WARNING: wrong pfam id format ($name)\n";
	}
	else{
		$pfam_acc = $1;
	}

	if(defined $id_regx){
		my $pfam_id = TCommon::get_pfamID_by_pfamACC($pfam_acc);
	
		unless($pfam_id){
			print STDERR "WARNING: no pfam id for $pfam_acc\n";
			return 1;
		}
		   
		return 1 unless $pfam_id =~ /$id_regx/;

		print STDERR "Matching family: $pfam_id\n";
	}

	if(defined $go_term){

	my $pfam_id = TCommon::get_pfamID_by_pfamACC($pfam_acc);

	unless($pfam_id){
		print STDERR "WARNING: no pfam id for $pfam_acc\n";
		return 1;
	}

	my $go_terms =  TCommon::get_go_by_pfam_name($pfam_id);

	return 1 unless @$go_terms;

	my $has_go = 0;
	foreach my $g (@$go_terms){
		my ($term,$class) = ($g->{term},$g->{class});

		if($term eq $go_term){
			$has_go = 1;
			last;
		}
	}

	return 1 unless $has_go;
	}

	
	my ($pfam_hmm_acc,$query_name);
	#die "ERROR: wrong name format ($name)\n" unless $name =~ /^(\S+?)\=\+\=(\S+)$/;


	unless ( ($pfam_hmm_acc,$query_name) = split(/\=\+\=/ , $name) ) {

		#unless (($pfam_hmm_acc,$query_name, $evalue) = $name =~ /^(\S+?)\=\+\=(\S+)\=\+\=(\S+)$/) {
		
		print STDERR "ERROR: wrong name format ($name)\n";
		print STDERR "ERROR: Expected something like: \"PF06147.3=+=259156_1787_1079/1_1_1\"\n";
		print STDERR "ERROR: Or: \"PF06147.3=+=259156_1787_1079/1_1_1=+=1.8e-09\"\n";
		die;
		
	}

	#print STDERR "Got: $pfam_hmm_acc,$query_name, $evalue\n";


	unless(exists $seqs_with_pfam_hit->{$pfam_hmm_acc}){
		$seqs_with_pfam_hit->{$pfam_hmm_acc} = [];
	}

	

	push @{ $seqs_with_pfam_hit->{$pfam_hmm_acc} }, [$query_name,$seq];
	
	return 1;
}





#########################
# submit jobs to creconstruct trees in parallel or serial
#########################
sub _submit_predict{
	my ($job_families,$outfile, $job_id, $scheduler) = @_;
	# job: $job_families is a subset of all pfam families
	# $job_families: array of tuples; each tuple: [acc, array]; array->array of tuples: [query_name,qseq]


	if(USE_CODINE){

		my $tmp_fasta_sequences="";
	
		foreach my $job_info (@$job_families){
			my ($pfam_hmm_acc,$hitting_egts) = @$job_info;
	
			foreach my $hs (@$hitting_egts){
			
				my ($query_name,$query_seq, $evalue);
				if (@$hs == 2) {
					($query_name,$query_seq) = @$hs;
					$tmp_fasta_sequences .= ">$pfam_hmm_acc=+=$query_name\n$query_seq\n";
				} else {
					($query_name,$query_seq, $evalue) = @$hs;
					$tmp_fasta_sequences .= ">$pfam_hmm_acc=+=$query_name=+=$evalue\n$query_seq\n";
				}

				
			}
		}
	
		

		require
			 Scheduler::DRMAA;	
		
		$scheduler->add_input_data($job_id, $tmp_fasta_sequences);

	}
	# use user defined scheduler
	# to submit job
	else{	

	
		my ($fh, $tmp_fasta) = tempfile(UNLINK => 0, DIR => TMP_DIR, SUFFIX => '.job.egt'); # has to be deleted by the job!
	
		foreach my $job_info (@$job_families){
			my ($pfam_hmm_acc,$hitting_egts) = @$job_info;
	
			foreach my $hs (@$hitting_egts){
				my ($query_name,$query_seq, $evalue);
				if (@$hs == 2) {
					($query_name,$query_seq) = @$hs;
					print $fh ">$pfam_hmm_acc=+=$query_name\n$query_seq\n";
				} else {
					($query_name,$query_seq, $evalue) = @$hs;
					print $fh ">$pfam_hmm_acc=+=$query_name=+=$evalue\n$query_seq\n";
				}

			}
		}
	
		close($fh);

		#print "Outfile: $outfile\n";
		my($filename, $directories, $suffix) = fileparse($outfile, qr/\.[^.]*/);


		my $error_file=$directories."/".$filename.".log";

		my $cmd = "$INSTALLDIR"."/bin/carma_classify.pl -j". ($main::opt_t?" -t":"") . ($main::opt_v?" -v":"") . " -o $outfile $tmp_fasta 2>> $error_file"; 
	#print "call: $cmd\n";
		
		$cmd = SCHEDULER_SUBMIT . " $cmd";
		unless(system($cmd) == 0){
			print STDERR "ERROR while submitting job\n($cmd)\n";
			exit 1;
		}
		

	return 1;
	}
}


##############################################
#
# Predict taxonomic origin of genomic DNA fragments
#
##############################################
sub carma_phylo{ 
	my $fasta_file;
	my $single_seqs_with_pfam_hit;
	($INSTALLDIR, $fasta_file, $single_seqs_with_pfam_hit) = @_;
	


	

		######################################
		# reconstruct trees in parallel or serial (calls JOBs!)
		######################################
	
		
		unless($main::opt_o){
			main::usage();
			print STDERR "ERROR: -o option missing!\n";
			exit 1;
		}


		unless(-e $main::opt_o){
			main::usage();
			print STDERR "ERROR (reconstruct trees in parallel): $main::opt_o directory does not exist!\n";
			exit 1;
		}

		unless(-d $main::opt_o){
			main::usage();
			print STDERR "ERROR (reconstruct trees in parallel): $main::opt_o is not a directory!\n";
			exit 1;
		}



		TCommon::check_pfam_version();
		

		my $tmp = $fasta_file;
		$tmp =~ s/\/$//;

		my @tmpar = split(/\//,$tmp);
		my $outdir = pop @tmpar;

		$outdir = "$main::opt_o/$outdir";


		unless(-d $outdir){
			unless(mkdir($outdir)){
				print STDERR "ERROR (reconstruct trees in parallel): Could not create directory $outdir!\n";
				exit 1;
			}
		}
		
		
		unless(-d DIST_CACHE_DIR) {
			print STDERR "ERROR:(DIST_CACHE_DIR) directory ".DIST_CACHE_DIR." does not exist!\n";
			exit 1;
		}
		
		
		##################################################
		
		
#		if(USE_CODINE){
#			 require
#			   Scheduler::Codine;

#			# Submit jobs on idle queue
#			if($main::opt_i){
#				print "Submitting jobs to idle queue ....\n";
#				Scheduler::Codine->options('-l idle=1 ' . CODINE_OPTIONS);
#			}
#			else{
#				Scheduler::Codine->options(CODINE_OPTIONS);
#			}

#			# freeze Codine in order to submit array job
#			Scheduler::Codine->freeze();
#		}
	
		my $pfam_family_number;
		# temporary hash to count pfam families:
		my $pfam_families={};


		$pfam_family_number = keys %$single_seqs_with_pfam_hit;

		
		if (! $pfam_family_number) {
			print STDERR "ERROR: No EGTs found ! (Maybe the reads did not contain EGTs?)\n";
			die;
		}
		
		my $family_per_job=1;
	
				
		unless (defined $main::opt_c) {
			$main::opt_c=100;
		}
		
		if ($main::opt_c < 1) {
			die "Option -c has to be >= 1\n";
		}
		$family_per_job =$pfam_family_number / $main::opt_c ;

		print STDERR "Reads have hits to $pfam_family_number different pfam families\n";
		
#		if (USE_CODINE) {
		
		print STDERR "Submitting $main::opt_c jobs, $family_per_job pfam famil". (($family_per_job==1)?"y":"ies")  ." per job ... \n";
		
#		} else {
#			print STDERR "$pfam_family_number jobs to be called ... \n";
#		}

		
		my @submit = ();

		my $prefix;
	
		if ($main::opt_c) {
			$prefix = $main::opt_c;
		} else {
			$prefix = "0";  # or = $pfam_family_number
		}




		my $scheduler;
		my $tmp_dir;
		if(USE_CODINE){
			
			require
				 Scheduler::DRMAA;		
			
				
			my @job_cmd_array;
			push(@job_cmd_array, $INSTALLDIR."/bin/carma_classify.sh" );
		 	push(@job_cmd_array, "-j" );
		 	if (defined $main::opt_t) {
		 		push(@job_cmd_array, "-t" );
		 	}
			
		 	if (defined $main::opt_v) {
		 		push(@job_cmd_array, "-v" );
		 	}

			
			$tmp_dir = FileUtils::get_tmp_dir_object(TMP_DIR);

			my $errorfile="$outdir/${prefix}_".$Scheduler::DRMAA::DRMAA_PLACEHOLDER_INCR.".log";
			
			
			$scheduler = Scheduler::DRMAA->new(temp_dir => $tmp_dir->dirname, commandline => [@job_cmd_array], error_file => $errorfile );
			
			
			
			if ($main::opt_i) {
		 			$scheduler->set_native_option('-l idle=1 ' . CODINE_OPTIONS);
			} else {
				$scheduler->set_native_option(CODINE_OPTIONS);
			}
			
			
		}



		my $counter = 1; 
		my $submitted = 0;
		


		my @job_families = ();

		my $hitting_egts;
		my $pfam_hmm_acc;
		########################
		# Submit one job for $family_per_job Pfam famlies
		########################

		

		foreach $pfam_hmm_acc (keys %$single_seqs_with_pfam_hit ) {

			$hitting_egts = [];

			push @$hitting_egts, @{$single_seqs_with_pfam_hit->{$pfam_hmm_acc}}; # array of tuples [query_name, seq]
			push @job_families, [$pfam_hmm_acc,$hitting_egts];

			# submit new job for @job_families
			if(@job_families >= $family_per_job){ #otherwise continue filling @job_families
				
				
				my $outfile = "$outdir/${prefix}_$counter.out";
				
				if( (! -e $outfile) or (-z $outfile) ){
					$submitted++;
	
					#my $family_number = @job_families;



					_submit_predict(\@job_families,$outfile, $counter, $scheduler);			
				} else {
					print STDERR "File \"$outfile\" already exists, skip it. If you want to recreate it, delete the old one first.\n";
				}

				@job_families = ();
				$counter++;
			}
		}

		# submit the last families
		if(@job_families){
			#$counter++;

			my $outfile = "$outdir/${prefix}_$counter.out";

			if( (! -e $outfile) or (-z $outfile) ){
				$submitted++;		

				_submit_predict(\@job_families,$outfile, $counter, $scheduler);
			}
			else{
				print STDERR "File \"$outfile\" already exists, skip it. If you want to recreate it, delete the old one first.\n";
			}
		}


		if(USE_CODINE && $submitted > 0){
			$scheduler->submit();


			print STDERR "Done.\nSubmitted $submitted jobs!\n";
				
			my @error_jobs;
				
			my $success_jobs = 0;
			print STDERR "waiting and parsing results...\n";
			$scheduler->iterate(
				sub {
					my ($name, $fh) = @_;
					#print "Name: $name\n";
					#return unless ($all_jobs_ok);
				
					my $outfile = "$outdir/${prefix}_$name.out";
					my $out_fh;
					unless(open($out_fh,">$outfile")){
						print STDERR "ERROR: Could not open file $outfile:\n$!\n";
						die;
					}
				
					while (<$fh>) {
						print $out_fh $_;
					}
					close ($out_fh);
					
					$success_jobs++;
					print STDERR "Job \"" . $name . "\" finished successfully.\n";
					if ($submitted != (@error_jobs+$success_jobs)) {
						print STDERR ($submitted-(@error_jobs+$success_jobs)) . " are left.\n";
					}
				
				},
				sub {
					printf STDERR "Job %s failed: %s\n",$_[0],$_[1];

					push (@error_jobs, $_[0]);
					
				});
				
			if (@error_jobs > 0) {
				print STDERR "Sorry, some jobs did fail:\n";
				print STDERR "Jobs that failed: \"" . join(',', @error_jobs ) . "\"\n";
				print STDERR "Simply rerun CARMA with same parameters, only jobs that failed will be resubmitted!\n";
				print STDERR "(Will not work if called via carma.pl using option \"-o\"!)\n";
				exit(1);
			}
			
		}
		exit 0;
	


	
}

