package TCarma_phylo_job;
#
#  Copyright (C) 2007 CeBiTec, Bielefeld University
#  Written by Wolfgang Gerlach.
#  The original version (CARMA 1.2) was developed by Lutz Krause.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  version 2 as published by the Free Software Foundation.
#
#  This file is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this file; see the file LICENSE.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.
#

use strict;
use warnings;

#use lib '/homes/wgerlach/carma/share/perl';

#use Getopt::Std;
#use File::Basename;
#use POSIX qw(tmpnam);
use File::Temp qw(tempfile tempdir unlink0);
use File::Path qw(rmtree);
#use File::Temp qw/ tempfile tempdir /;

use DBI;
use TCommon;
use TCarma_MatePairFunctions;


use Tree;
use Bio::SearchIO;
use Cwd;
use IO::Handle;

use DNAUtils;

use TConfigs;
use TCarma_phylo;

1;


#############################
# Add sequence to multiple alginment 
# using hmmalign from the hmmer package
#############################
sub _hmmalign_align{

	my ($hmm_file,$egt_fasta_file,$pfam_alignment) = @_;

	#my $hmmalign_out = TMP_DIR . basename(tmpnam());
	#my ($fh, $hmmalign_out) = tempfile (UNLINK => $main::opt_t?0:1,  DIR =>  TMP_DIR, OPEN=>0, SUFFIX => ".hmmalignout");
	my ($fh, $hmmalign_out) = tempfile (OPEN=>0, SUFFIX => ".hmmalignout");
	
	unless ($hmm_file) {
		print STDERR "*hmm_file: $hmm_file\n";
		print STDERR "fasta_file: $egt_fasta_file\n";
		print STDERR "pfam_alignment: $pfam_alignment\n";
		die;
	}
	
	unless ($egt_fasta_file) {
		print STDERR "hmm_file: $hmm_file\n";
		print STDERR "*fasta_file: $egt_fasta_file\n";
		print STDERR "pfam_alignment: $pfam_alignment\n";
		die;
	}
	
	unless ($pfam_alignment) {
		print STDERR "hmm_file: $hmm_file\n";
		print STDERR "fasta_file: $egt_fasta_file\n";
		print STDERR "*pfam_alignment: $pfam_alignment\n";
		die;
	}

	my $cmd = HMMALIGN." --oneline --withali $pfam_alignment $hmm_file $egt_fasta_file > $hmmalign_out";

	unless(system($cmd) == 0){
		unless ($main::opt_t) {
			unless (unlink0($fh, $hmmalign_out)) {
				print STDERR "ERROR: (_hmmalign_align) Error unlinking file $hmmalign_out safely";
			}	
		}
		print STDERR "ERROR while running \"" . $cmd . "\":\n";
		print STDERR "$!\n";
		die;
	}
	
	unless (-e $hmmalign_out) {
		print STDERR "ERROR (_hmmalign_align): File $hmmalign_out has not been written!?\n";
		exit(1);
	}


	return $hmmalign_out;
}

######################################
# align DNA fragments to multiple alignment of hitting Pfam families
######################################
sub run_hmmalign{
	my ($single_seqs_with_pfam_hit,$verbose) = @_;
	
	print STDERR "Aligning DNA fragments to Pfam multiple alignments ... " if $verbose;

	my ($pfam_hmm_acc,$hitting_egts);
	
	my @alignments = ();

	# each family:
	while ( ($pfam_hmm_acc,$hitting_egts) = each %$single_seqs_with_pfam_hit) {
		unless(@$hitting_egts){
			print STDERR "ERROR(run_hmmalign): Empty hitting_seq!?\n";
			die;
			#next;
		}
	
		my $pfam_acc = $pfam_hmm_acc;
	
		$pfam_acc =~ s/\.\d+$//;
	
		my $pfam_alignment = PFAM_FAMILY_ALIGNMENTS . "/$pfam_acc.fas";
	
		unless(-e $pfam_alignment){
			print STDERR "ERROR(run_hmmalign): File $pfam_alignment does not exist!\n" if WARNINGS; # TODO ????
			next;
		}
	
		my $num_lines=`head $pfam_alignment | wc -l`; # TODO neighbor needs at least 3 sequences, do something better than this work-around.
		if ($num_lines <= 2) {
			print STDERR "skipped $pfam_alignment, too few members.\n";
			next;
		}
		
		my $member_count = 0;
	
		die "ERROR(run_hmmalign) while parsing pfam alignment\n" unless &TCommon::parse_fasta($pfam_alignment, 
											sub { ${ $_[2] }++; return 1;},
											 \$member_count);    #simply increases member_count by one! TODO: necessary??
	
		my $hitting_egt_number = @$hitting_egts;

		print STDERR "$pfam_hmm_acc family members: $member_count, matching fragments: $hitting_egt_number\n" if DEBUG;


#print "$pfam_hmm_acc family members: $member_count, matching fragments: $hitting_egt_number\n";

		my @hitting_egts_partioning_20p = ();

		my $member_20p = $member_count * 0.2;
	


	
	
		# bucket number:
		my $j = 0;
		# each egt of a family:
		for(my $i = 0; $i < @$hitting_egts; $i++){
			push  @{ $hitting_egts_partioning_20p[$j] }, $hitting_egts->[$i];
			
#			#push also mate if available and delete it from hitting_egts:
#			if ($main::opt_m) {  # }
#				my ($mateName) = TCarma_MatePairFunctions::getMateName($hitting_egts->[$i]->[0]);
#				$mateName=$$mateName; # TODO isn't there a proper way?
#			
#				# illumina format: "HWI-EAS217_1_2013P:1:95:916:131/2_1_2"
#				for(my $findMatePos = $i+1; $findMatePos < @$hitting_egts; $findMatePos++){
#					my ($currentEGT) = $hitting_egts->[$findMatePos]->[0] =~ /^(\S+\d)_-?\d_-?\d$/;
#					if ($currentEGT eq $mateName) {
#						# found mate.
#						# insert into hitting_egts_partioning_20p[$j]:
#						push  @{ $hitting_egts_partioning_20p[$j] }, $hitting_egts->[$findMatePos];
#						# delete mate from $hitting_egts:
#						splice (@{$hitting_egts},$findMatePos,1);
#						last; #leave for-loop
#					}
#				}
#				
#			}				
			unless(@{ $hitting_egts->[$i]  }){
				print STDERR "ERROR(run_hmmalign): ups, hitting_egts is emtpy!!!!\n"; 
				die;
			}

			if( @{ $hitting_egts_partioning_20p[$j] } >= $member_20p){
				$j++; #next bucket
			}
		} # end of for
		
		foreach my $hitting_egts_20p (@hitting_egts_partioning_20p){
		
			unless(@$hitting_egts_20p){
				my $e = @hitting_egts_partioning_20p;
				print STDERR "ERROR(run_hmmalign): empty hitting_egts_20p! ($member_20p,$member_count,$e)\n";
				next;
			}

			# create tmp alignment not containing query sequences		
			my $hmm_file = main::get_hmmfile($pfam_hmm_acc);
		
			my ($PR, $tmp_egts) = tempfile (UNLINK => $main::opt_t?0:1, SUFFIX => ".20.egt");
			
			foreach my $hitting_seq (@$hitting_egts_20p){		
				my ($query_name,$query_seq) = @$hitting_seq;
				print $PR ">c_$query_name\n$query_seq\n";
			}  
		

			$PR->close;

			unless(-e $tmp_egts){
				print STDERR "ERROR(run_hmmalign): fasta file $tmp_egts does not exist:\n$!\n";
				die;
			}		

			if (-z $tmp_egts){
				print STDERR "ERROR(run_hmmalign): fasta file $tmp_egts is of size zero:\n$!\n";
				die;
			}		

		
			unless ($hmm_file) {
				print STDERR "*hmm_file: $hmm_file\n";
				print STDERR "tmp_egts: $tmp_egts\n";
				print STDERR "pfam_alignment: $pfam_alignment\n";
				die;
			}
		
			unless ($tmp_egts) {
				print STDERR "hmm_file: $hmm_file\n";
				print STDERR "*tmp_egts: $tmp_egts\n";
				print STDERR "pfam_alignment: $pfam_alignment\n";
				die;
			}
			unless ($pfam_alignment) {
				print STDERR "hmm_file: $hmm_file\n";
				print STDERR "tmp_egts: $tmp_egts\n";
				print STDERR "*pfam_alignment: $pfam_alignment\n";
				die;
			}
		
			my $alignment; #is the filename that contains the hmmalign output
			eval{		
				$alignment = _hmmalign_align($hmm_file,$tmp_egts,$pfam_alignment);		
			};
			
			unlink($hmm_file) unless ($main::opt_t);
			
			# Remove tmp files if an error occured
			if($@){
				unless ($main::opt_t) {
					unlink0($PR, $tmp_egts) or die "ERROR: (run_hmmalign) Error unlinking file $tmp_egts safely";   #TODO does not work!?
				}
				print STDERR "\nERROR(run_hmmalign): $@\n($main::OUTFILE)\n";
				die;
			}
			print STDERR "Done.\n" if DEBUG;
		
			unlink $tmp_egts unless ($main::opt_t);
			#File::Temp::unlink0($PR, $tmp_egts) or die "ERROR: (run_hmmalign) Error unlinking file $tmp_egts safely"; #TODO does not work!?
			#unlink $tmp_align if $tmp_align;
		
			push @alignments,[$pfam_acc,$alignment];
		
			print STDERR "Alignment for $pfam_acc: $alignment\n" if DEBUG;		
		}
	}
	print STDERR "Done.\n" if $verbose;
	return \@alignments;
}



##################################################
# Reconstruct trees locally   (Jobs!!)
##################################################
	
sub carma_phylo_job{

	my ($single_seqs_with_pfam_hit) = @_;


	# align DNA fragments to multiple alignment of hitting Pfam families
	# an array of tuples [acc, filename of hmmout], acc is not unique.

	my $alignment_files = &run_hmmalign($single_seqs_with_pfam_hit,$main::opt_v);

	
	unless(ref $alignment_files){
		print STDERR "ERROR (Reconstruct trees locally): Could not align seq to pfam family!\nBailing out ...\n";
		exit 1;
	}
	
	my %detail = ();
	my $tree_number = 0;
	my %predicted_taxa = ();
 
	my $outfile_workingname="";
 	if ($main::opt_o) {
		$outfile_workingname=$main::opt_o . ".part";

		unless(open(TAXA_OUT,">$outfile_workingname")){
			print STDERR "ERROR (Reconstruct trees locally): Could not open file $main::opt_o:\n$!\n";
			die;
		}
	}

	################################################################
	# Reconstruct phylogenetic tree from each multiple alignment
	################################################################
	my $anumber;
	if($main::opt_v){
		$anumber = @$alignment_files;
	}

	my %precomputed_distances = ();

	my $current = 0;
	foreach my $alignment_info (@$alignment_files){
		my ($pfam_acc,$alignment) = @$alignment_info;

		$current++;

		print STDERR "Processing alignment for family $pfam_acc $current/$anumber\n" if $main::opt_v;

		my $tmp_dir = tempdir ( DIR =>  TMP_DIR, CLEANUP => (($main::opt_t)?0:1));
		my ($fh , $matrix_tmp_file) = tempfile(UNLINK => ($main::opt_t?0:1), OPEN => 0, SUFFIX => '.mtx');
		my ($fh2, $out_acc_tmp)     = tempfile(UNLINK => ($main::opt_t?0:1), OPEN => 0, SUFFIX => '.acc');   # same as $acc_by_id_file
		

		my $matrix_created = 0;

		######################################
		# create pairwise distance matrix
		######################################
#my $create_matrix_time = time();

		eval{
			$matrix_created = TCommon::create_matrix(\%precomputed_distances,
									$pfam_acc,
									$alignment,
									'stockholm',
									$matrix_tmp_file,
									1,
									$out_acc_tmp,
									1,
									0,
									$main::opt_v
									);
		};
#print STDERR "create_matrix_time: ". (time() - $create_matrix_time) ."\n";

		print STDERR "Created matrix $matrix_tmp_file\n" if $main::opt_v;

		#################################
		# Clean up if an error occured
		#################################
		if($@ or ! $matrix_created){	
			unlink($alignment) unless ($main::opt_t) ;
			unlink($matrix_tmp_file) unless ($main::opt_t) ;
			unlink($out_acc_tmp) unless ($main::opt_t) ;
			print STDERR "ERROR: $@\n" if $@;
		
			rmtree($tmp_dir) unless ($main::opt_t);
			
			die if $@;
			exit 1;
		}

		# remove tmp file
		unlink $alignment unless ($main::opt_t) ;

		#######################
		# reconstruct phylogenetic tree using neighbor-joining
		#######################
#my $run_phylip_nj_time = time();
		my $tmp_tree;
		eval{
			$tmp_tree = TCommon::run_phylip_nj($matrix_tmp_file,$tmp_dir,$main::opt_v);
		};

#print STDERR "run_phylip_nj_time: ". (time() - $run_phylip_nj_time) ."\n\n";

	
		########################################
		# remove tmp files if an error occured
		########################################
		if($@ or ! $tmp_tree){
			unlink($matrix_tmp_file) unless ($main::opt_t) ;
			unlink($out_acc_tmp) unless ($main::opt_t) ;
			print STDERR "\nERROR: $@\n" if $@;
		
			my $cmd = "rm -r $tmp_dir";
			unless ($main::opt_t) {
				unless(system($cmd) == 0){
					print STDERR "WARNING(2): Could not remove dir $tmp_dir:\n$!\n";
				}
			}
			die if $@;
			exit 1;
		}
		unlink($matrix_tmp_file) unless ($main::opt_t) ;
	
		unless($tmp_tree){
			exit 1;
		}

		$tree_number++;

		#############################################
		# predict taxonomic origin of each DNA fragment 
		# based on phylogenetic relationships
		#############################################
#my $predict_taxa_time = time();		
		my $taxa = TCommon::predict_taxa($tmp_tree,
						 $out_acc_tmp,
						 $main::opt_v,
						 $alignment);
#print STDERR "predict_taxa_time: ". (time() - $predict_taxa_time) ."\n";	


		# add predicted taxa to taxonomic profile
		foreach my $ti (@$taxa){
				my ($q,$t) = @$ti;
				
				$q =~ s/^c_//;	# delete "c_" in front read description
				$t =~ s/\;//g;
				
				my $sep=SEPSYMBOL;
				my ($last_ncbi_code) = $t =~ /$sep(\d+)$/;
				unless ($last_ncbi_code) {
					$last_ncbi_code=0;
				}
				
				
				
				my $go_ids_array =  TCommon::get_go_id_by_pfam_acc($pfam_acc);
				my $go_ids = "{".join(",",@$go_ids_array)."}";
				
				my $output_line="$q\t$pfam_acc\t$go_ids\t$last_ncbi_code";
				
				if (lc(CONSIDERED_RANKS_MODUS) ne 'none') {
					my $taxon = TCommon::get_taxon_for_ncbi_code($last_ncbi_code);
					$output_line.="\t$taxon";
				}
				$output_line.="\n";
				
				
				if ($main::opt_o) {
					print TAXA_OUT $output_line;
				} else {
					print STDOUT $output_line;
				}
		}

		####################
		# remove tmp files
		####################
		unlink($out_acc_tmp) unless ($main::opt_t) ;	

			
		opendir(INDIR,$tmp_dir) || die $!;
		my @a=readdir(INDIR);
		close INDIR;
		
		if (-d $tmp_dir) { #due to nfs system with lock-files. TODO this is not a perfect solution...
			
			foreach my $file (@a) {
				unlink("$tmp_dir/$file") unless ($main::opt_t) ;
			}
		}

	} # end of "foreach my alignment_info"
	if ($main::opt_o) {
		close(TAXA_OUT);
		#now rename the file 
		unless (rename($outfile_workingname, $main::opt_o)) {
			print STDERR "ERROR: Could not rename $outfile_workingname to ". $main::opt_o ." !?\n";
		}
		
	}
	
	return 0;
}
