package TConfigs;

#
#  Copyright (C) 2007 CeBiTec, Bielefeld University
#  Written by Wolfgang Gerlach.
#  The original version (CARMA 1.2) was developed by Lutz Krause.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  version 2 as published by the Free Software Foundation.
#
#  This file is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this file; see the file LICENSE.  If not, write to
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#  Boston, MA 02111-1307, USA.
#
use strict;
use warnings;

# check if perl version is sufficient
use 5.8.3;

use base qw(Exporter);

# Warnings will be printed if set to true
use constant WARNINGS => 1;

# If set to true, taxonomic origins 
# will be predicted up to the rank 
# of species
use constant GET_SPECIES => 1;

# Print information for
# debugging purpose
use constant DEBUG => 0;

# Minimum length of Pfam hits
# The taxonomic origin will be predicted only for
# hits (EGTs) longer than this value (default: 25)
use constant MINIMUM_PFAM_HIT_LENGTH => 10;

use constant MAX_FAMILY_SIZE => 3000;

# default: 10
use constant BLAST_DEFAULT_EVALUE => 10;

# default: 0.01
use constant HMMPFAM_DEFAULT_EVALUE => 0.01;


use constant SEPSYMBOL => '!';


# The Pfam version of the files (Pfam_fs, Pfam-A.full, Pfam-A.fasta) e.g. '23.0'
# If somebody else maintains for you the mysql-Pfam database (like on the Bielefeld cebitec
# system), it can happen that the mysql database is updated without your knowledge and the
# files (Pfam_fs, Pfam-A.full, Pfam-A.fasta) are from the old Pfam version. Therefore CARMA
# now compares this entry with the version number given in the "VERSION" table in the mysql
# database. If you can guarantee the files and the mysql database are from the same Pfam 
# version you can disable this check with '-1'.
use constant PFAM_VERSION => '23.0';


#############################
# System parameters
#############################

# directory for (global) temporary files (e.g. /tmp/carma/)
# the jobs do store their temporary file in the default temp-directories of their machines (local)
use constant TMP_DIR => '/vol/codine-tmp/';

# directory to cache distance matrices for pfam families
# do not delete them as it take a long time to construct them
use constant DIST_CACHE_DIR => '/vol/cluster-data/carma/matrix'.PFAM_VERSION.'/';

use constant NAMES_FILE => '/vol/biodb/ncbi_taxonomy/names.dmp';
use constant NODES_FILE => '/vol/biodb/ncbi_taxonomy/nodes.dmp';
use constant MERGED_FILE => '/vol/biodb/ncbi_taxonomy/merged.dmp';

# The output of CARMA (the classification) contains the ncbi_id and a human readable taxon
# like (cellular organisms!Bacteria!Actinobacteria!Actinobacteria (class)!Actinobacteridae!Actinomycetales)
# for each classified EGT. Here you can define which taxonmic level you want to see.
# If the taxon does not contain the level you want to see, it will be skipped, so be careful with
# postprocessing! In general, we recommend to use the ncbi_id instead for postprocessing.
use constant CONSIDERED_RANKS => ('superkingdom'	=> 1,
					'phylum'	=> 1,
					'class'		=> 1,
					'order'		=> 1,
					'family'	=> 1,
					'genus'		=> 1,
					'species'	=> 1,
					'tribe'		=> 0,
					'forma'		=> 0,
					'kingdom'	=> 0,
					'subclass'	=> 0,
					'varietas'	=> 0,
					'subkingdom'	=> 0,
					'suborder'	=> 0,
					'infraorder'	=> 0,
					);
					
# 'none' show no human readable taxon in the output (saves space)
# 'all' show the complete taxon with all known taxonomic levels
# 'user' show only taxonmic levels as defined in CONSIDERED_RANKS
use constant CONSIDERED_RANKS_MODUS => 'user';

# The Pfam_fs profile HMM dabase
use constant PFAM_FS => '/vol/biodb/pfam/Pfam_fs';


# Fasta version of PFAM-A database (Pfam-A.fasta)
use constant PFAM_A_FASTA => '/vol/biodb/pfam/Pfam-A.fasta';


# Empty directory in which processed alignments of each Pfam family are stored
# (when running process_alignments the processed alignments will be saved in this
#  directory)
use constant PFAM_FAMILY_ALIGNMENTS => '/vol/cluster-data/carma/db_pfam_v'.PFAM_VERSION.'/alignments/';


# blastall executable
use constant BLASTALL => "blastall";

# hmmpfam executable
# hmmpfam is part of the hmmer package
use constant HMMPFAM => "hmmpfam";

# hmmfetch executable
# hmmfetch is part of the hmmer package
use constant HMMFETCH => "hmmfetch";

# hmmalign exectable
# hmmalign is part of the hmmer package
use constant HMMALIGN => "hmmalign";

# The neighbor executable from the Phylip package
use constant PHYLIP_NJ_EXEC => 'neighbor';


# use Codine (Sun Grid Engine) to submit jobs to a
# high computing resource
# Set to '0' if Codine is not
# available
use constant USE_CODINE => 1;

# Codine options
use constant CODINE_OPTIONS => '-l arch=sol-amd64';
#use constant CODINE_OPTIONS => '-q \'*@@qics\' -V';
#use constant CODINE_OPTIONS => '-q \'*@@i386hosts\' -V';


# If codine is not available, this program
# can be used to submit jobs on high computing resource
# e.g. 'qsub -V'
use constant SCHEDULER_SUBMIT => '';

# Name of Pfam MySQL database
use constant PFAM_MYSQL_DATABASE => 'pfam';
#use constant PFAM_MYSQL_DATABASE => 'pfam22';

# Database host for Pfam MySQL database
use constant DBHOST => 'dbhost';

# MySQL user login to access
# Pfam MySQL database
use constant MYSQL_USER => 'anonymous';

# MySQL password to access
# Pfam MySQL database
use constant MYSQL_PASSWD => '';


##########################
# End system parameters
##########################

use vars qw (
	     @EXPORT
	     );

@EXPORT = qw(
	     USE_CODINE
	     SCHEDULER_SUBMIT
	     MYSQL_PASSWD
	     DEBUG
	     PFAM_MYSQL_DATABASE
	     MYSQL_USER
	     DBHOST
	     MYSQL_USER
	     PHYLIP_NJ_EXEC
	     CODINE_OPTIONS
	     HMMALIGN
	     HMMFETCH
	     HMMPFAM
	     BLASTALL
	     TMP_DIR
	     PHYLIP_NJ_EXEC	     
	     WARNINGS
	     GET_SPECIES
	     MINIMUM_PFAM_HIT_LENGTH
	     PFAM_FS
	     PFAM_A_FASTA
	     PFAM_FAMILY_ALIGNMENTS
	     MAX_FAMILY_SIZE
	     BLAST_DEFAULT_EVALUE
	     HMMPFAM_DEFAULT_EVALUE
	     SEPSYMBOL
	     DIST_CACHE_DIR
             PFAM_VERSION
	     NODES_FILE
	     NAMES_FILE
             MERGED_FILE
	     CONSIDERED_RANKS
	     CONSIDERED_RANKS_MODUS
	     );




unless(-e TMP_DIR){
    unless( mkdir(TMP_DIR) ){
	print STDERR "ERROR: Could not create directory ",TMP_DIR,"\n";
	die;
    }
    system("chmod go+rwx ".TMP_DIR);
}



1;

